# Copyright (C) 2008 by Canonical Ltd

"""Pure Python (slower) implementation of miniobject."""

class _mini_meta_base(object):
    """Base metaclass for mini objects."""

    def __init__(self, name, instancevars):
        self._name = name
        self._instancevars = list(instancevars)

    def __repr__(self):
        return "%s(%r, %r)" % (
            self.__class__.__name__, self._name, self._instancevars)


class frozen_mini_class(_mini_meta_base):
    """Metaclass for mini objects."""
    
    def __call__(self, *args):
        """Construct a new instance"""
        i = _FrozenMiniInstance(self, zip(self._instancevars, args))
        return i


class frozen_indexed_mini_class(_mini_meta_base):

    def __call__(self, *args):
        """Construct a new instance"""
        i = _FrozenIndexedMiniInstance(self, zip(self._instancevars, args))
        return i


class _FrozenMiniBase(object):
    """An instance of an immutable miniobject.

    In this Python implementation, the fields are stored directly in 
    the instance dictionary of this object.
    """

    def __init__(self, cls, ivar_dict):
        self.__dict__['_cls'] = cls
        self.__dict__.update(ivar_dict)

    def __repr__(self):
        return "%s(%s)" % (
            self._cls._name,
            ", ".join(map(repr, self._get_values_tuple())))

    def _get_values_tuple(self):
        return tuple(getattr(self, k) for k in self._cls._instancevars)

    def __setattr__(self, k, v):
        # TypeError is what's raised by tuples
        raise TypeError("attribute assignment not supported by frozen_mini_class")

    def __eq__(self, other):
        return (isinstance(other, _FrozenMiniInstance)
            and other._cls == self._cls
            and other.__dict__ == self.__dict__)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self._cls._name) ^ hash(self._get_values_tuple())


class _FrozenMiniInstance(_FrozenMiniBase):

    def __getitem__(self, index):
        raise TypeError("getitem not supported by frozen_mini_class")

    def __len__(self):
        raise TypeError("len not supported by frozen_mini_class")



